@@ -119,7 +119,8 @@ class CommonView(APIView):
 
         # feedback
         feedback = Service(name='feedback', path='/feedback')
-        feedback.add_view('POST', 'feedback', klass=cls)
+        feedback.add_view('POST', 'feedback', klass=cls,
+                          permission='common.feedback')
         config.add_cornice_service(feedback)
 
 
