@@ -73,7 +73,7 @@ class APIMasterView(APIView):
         if prefix:
             return prefix
         model_name = cls.get_normalized_model_name()
-        return 'api.{}s'.format(model_name)
+        return '{}s'.format(model_name)
 
     @classmethod
     def get_permission_prefix(cls):
