@@ -51,3 +51,11 @@ class ConfigExtension(BaseExtension):
         # provide default theme selection
         config.setdefault('tailbone', 'themes', 'default, falafel')
         config.setdefault('tailbone', 'themes.expose_picker', 'true')
+
+
+def legacy_mobile_enabled(config):
+    return config.getbool('tailbone', 'legacy_mobile.enabled',
+                          default=True)
+
+def protected_usernames(config):
+    return config.getlist('tailbone', 'protected_usernames')
