@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -609,6 +609,7 @@ class Form(object):
             self.set_renderer(key, self.render_codeblock)
             self.set_widget(key, dfwidget.TextAreaWidget(cols=80, rows=8))
         elif type_ == 'text':
+            self.set_renderer(key, self.render_pre_sans_serif)
             self.set_widget(key, dfwidget.TextAreaWidget(cols=80, rows=8))
         elif type_ == 'file':
             tmpstore = SessionFileUploadTempStore(self.request)
@@ -903,6 +904,15 @@ class Form(object):
             return ""
         return HTML.tag('pre', value)
 
+    def render_pre_sans_serif(self, record, field_name):
+        value = self.obtain_value(record, field_name)
+        if value is None:
+            return ""
+        # this uses a Bulma helper class, for which we also add custom styles
+        # to our "default" base.css (for jquery theme)
+        return HTML.tag('pre', class_='is-family-sans-serif',
+                        c=value)
+
     def obtain_value(self, record, field_name):
         if record:
             try:
