@@ -210,7 +210,7 @@ class GridFilter(object):
             normalized = choices
         elif isinstance(choices, dict):
             normalized = OrderedDict([
-                (key, choices[value])
+                (key, choices[key])
                 for key in sorted(choices)])
         elif isinstance(choices, list):
             normalized = OrderedDict([
