@@ -32,11 +32,12 @@ from decimal import Decimal
 from rattail.time import localtime, make_utc
 from rattail.util import (pretty_quantity, pretty_hours, hours_as_decimal,
                           OrderedDict)
+from rattail.db.util import maxlen
 
 from webhelpers2.html import *
 from webhelpers2.html.tags import *
 
-from tailbone.util import csrf_token, pretty_datetime, raw_datetime
+from tailbone.util import csrf_token, get_csrf_token, pretty_datetime, raw_datetime
 
 
 def pretty_date(date):
