@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -172,18 +172,15 @@ class AuthenticationView(View):
         if not self.request.user:
             return self.redirect(self.request.route_url('home'))
 
-        if self.rattail_config.demo() and self.request.user.username == 'chuck':
-            self.request.session.flash("Cannot change password for 'chuck' in demo mode", 'error')
-            return self.redirect(self.request.get_referrer())
-
+        use_buefy = self.get_use_buefy()
         schema = ChangePassword().bind(user=self.request.user)
-        form = forms.Form(schema=schema, request=self.request)
+        form = forms.Form(schema=schema, request=self.request, use_buefy=use_buefy)
         if form.validate(newstyle=True):
             set_user_password(self.request.user, form.validated['new_password'])
             self.request.session.flash("Your password has been changed.")
             return self.redirect(self.request.get_referrer())
 
-        return {'form': form}
+        return {'form': form, 'use_buefy': use_buefy}
 
     def become_root(self):
         """
@@ -209,6 +206,8 @@ class AuthenticationView(View):
 
     @classmethod
     def defaults(cls, config):
+        rattail_config = config.registry.settings.get('rattail_config')
+        legacy_mobile = cls.legacy_mobile_enabled(rattail_config)
 
         # forbidden
         config.add_forbidden_view(cls, attr='forbidden')
@@ -216,14 +215,16 @@ class AuthenticationView(View):
         # login
         config.add_route('login', '/login')
         config.add_view(cls, attr='login', route_name='login', renderer='/login.mako')
-        config.add_route('mobile.login', '/mobile/login')
-        config.add_view(cls, attr='mobile_login', route_name='mobile.login', renderer='/mobile/login.mako')
+        if legacy_mobile:
+            config.add_route('mobile.login', '/mobile/login')
+            config.add_view(cls, attr='mobile_login', route_name='mobile.login', renderer='/mobile/login.mako')
 
         # logout
         config.add_route('logout', '/logout')
         config.add_view(cls, attr='logout', route_name='logout')
-        config.add_route('mobile.logout', '/mobile/logout')
-        config.add_view(cls, attr='mobile_logout', route_name='mobile.logout')
+        if legacy_mobile:
+            config.add_route('mobile.logout', '/mobile/logout')
+            config.add_view(cls, attr='mobile_logout', route_name='mobile.logout')
 
         # no-op
         config.add_route('noop', '/noop')
