@@ -71,6 +71,7 @@ class BatchMasterView(MasterView):
     Base class for all "batch master" views.
     """
     default_handler_spec = None
+    batch_handler_class = None
     has_rows = True
     rows_deletable = True
     rows_downloadable_csv = True
@@ -120,29 +121,43 @@ class BatchMasterView(MasterView):
         super(BatchMasterView, self).__init__(request)
         self.handler = self.get_handler()
 
-    def get_handler(self):
+    @classmethod
+    def get_handler_factory(cls, rattail_config):
         """
-        Returns a `BatchHandler` instance for the view.  All (?) custom batch
-        views should define a default handler class; however this may in all
-        (?)  cases be overridden by config also.  The specific setting required
-        to do so will depend on the 'key' for the type of batch involved, e.g.
-        assuming the 'vendor_catalog' batch:
+        Returns the "factory" (class) which will be used to create the batch
+        handler.  All (?) custom batch views should define a default handler
+        class; however this may in all (?) cases be overridden by config also.
+        The specific setting required to do so will depend on the 'key' for the
+        type of batch involved, e.g.  assuming the 'inventory' batch:
 
         .. code-block:: ini
 
            [rattail.batch]
-           vendor_catalog.handler = myapp.batch.vendorcatalog:CustomCatalogHandler
+           inventory.handler = poser.batch.inventory:InventoryBatchHandler
 
         Note that the 'key' for a batch is generally the same as its primary
         table name, although technically it is whatever value returns from the
         ``batch_key`` attribute of the main batch model class.
         """
-        key = self.model_class.batch_key
-        spec = self.rattail_config.get('rattail.batch', '{}.handler'.format(key),
-                                       default=self.default_handler_spec)
-        if spec:
-            return load_object(spec)(self.rattail_config)
-        return self.batch_handler_class(self.rattail_config)
+        # first try to figure out if config defines a factory class
+        model_class = cls.get_model_class()
+        batch_key = model_class.batch_key
+        spec = rattail_config.get('rattail.batch', '{}.handler'.format(batch_key),
+                                  default=cls.default_handler_spec)
+        if spec: # yep, so use that
+            return load_object(spec)
+
+        # fall back to whatever class was defined statically
+        return cls.batch_handler_class
+
+    def get_handler(self):
+        """
+        Returns a batch handler instance to be used by the view.  Note that
+        this will use the factory provided by :meth:`get_handler_factory()` to
+        create the handler instance.
+        """
+        factory = self.get_handler_factory(self.rattail_config)
+        return factory(self.rattail_config)
 
     def download_path(self, batch, filename):
         return self.rattail_config.batch_filepath(batch.batch_key, batch.uuid, filename)
@@ -228,7 +243,7 @@ class BatchMasterView(MasterView):
 
         g.set_enum('status_code', self.model_class.STATUS)
 
-        g.set_renderer('id', self.render_batch_id)
+        g.set_renderer('id', self.render_id_str)
 
         g.set_link('id')
         g.set_link('description')
@@ -238,9 +253,6 @@ class BatchMasterView(MasterView):
         g.set_label('rowcount', "Rows")
         g.set_label('status_code', "Status")
 
-    def render_batch_id(self, batch, column):
-        return batch.id_str
-
     def template_kwargs_index(self, **kwargs):
         route_prefix = self.get_route_prefix()
         if self.results_executable:
@@ -270,7 +282,7 @@ class BatchMasterView(MasterView):
 
         # id
         f.set_readonly('id')
-        f.set_renderer('id', self.render_batch_id)
+        f.set_renderer('id', self.render_id_str)
         f.set_label('id', "Batch ID")
 
         # created
@@ -583,22 +595,20 @@ class BatchMasterView(MasterView):
     def configure_row_grid(self, g):
         super(BatchMasterView, self).configure_row_grid(g)
 
+        g.set_sort_defaults('sequence')
+        g.set_link('sequence')
+        g.set_label('sequence', "Seq.")
+        if 'sequence' in g.filters:
+            g.filters['sequence'].label = "Sequence"
+
         if 'status_code' in g.filters:
             g.filters['status_code'].set_value_renderer(grids.filters.EnumValueRenderer(self.model_row_class.STATUS))
 
-        g.set_sort_defaults('sequence')
-
         if self.model_row_class:
             g.set_enum('status_code', self.model_row_class.STATUS)
 
         g.set_renderer('status_code', self.render_row_status)
 
-        g.set_label('sequence', "Seq.")
-        g.set_label('status_code', "Status")
-        g.set_label('item_id', "Item ID")
-
-        g.set_link('sequence')
-
     def get_row_status_enum(self):
         return self.model_row_class.STATUS
 
@@ -736,7 +746,7 @@ class BatchMasterView(MasterView):
         """
         Delete all data (files etc.) for the batch.
         """
-        self.handler.delete(batch)
+        self.handler.do_delete(batch)
         super(BatchMasterView, self).delete_instance(batch)
 
     def get_fallback_templates(self, template, mobile=False):
@@ -1479,12 +1489,14 @@ class BatchMasterView(MasterView):
 
     @classmethod
     def _batch_defaults(cls, config):
+        rattail_config = config.registry.settings.get('rattail_config')
         model_key = cls.get_model_key()
         route_prefix = cls.get_route_prefix()
         url_prefix = cls.get_url_prefix()
         permission_prefix = cls.get_permission_prefix()
         model_title = cls.get_model_title()
         model_title_plural = cls.get_model_title_plural()
+        legacy_mobile = cls.legacy_mobile_enabled(rattail_config)
 
         # TODO: currently must do this here (in addition to `_defaults()` or
         # else the perm group label will not display correctly...
@@ -1528,14 +1540,16 @@ class BatchMasterView(MasterView):
                         permission='{}.edit'.format(permission_prefix))
 
         # mobile mark complete
-        config.add_route('mobile.{}.mark_complete'.format(route_prefix), '/mobile{}/{{{}}}/mark-complete'.format(url_prefix, model_key))
-        config.add_view(cls, attr='mobile_mark_complete', route_name='mobile.{}.mark_complete'.format(route_prefix),
-                        permission='{}.edit'.format(permission_prefix))
+        if legacy_mobile:
+            config.add_route('mobile.{}.mark_complete'.format(route_prefix), '/mobile{}/{{{}}}/mark-complete'.format(url_prefix, model_key))
+            config.add_view(cls, attr='mobile_mark_complete', route_name='mobile.{}.mark_complete'.format(route_prefix),
+                            permission='{}.edit'.format(permission_prefix))
 
         # mobile mark pending
-        config.add_route('mobile.{}.mark_pending'.format(route_prefix), '/mobile{}/{{{}}}/mark-pending'.format(url_prefix, model_key))
-        config.add_view(cls, attr='mobile_mark_pending', route_name='mobile.{}.mark_pending'.format(route_prefix),
-                        permission='{}.edit'.format(permission_prefix))
+        if legacy_mobile:
+            config.add_route('mobile.{}.mark_pending'.format(route_prefix), '/mobile{}/{{{}}}/mark-pending'.format(url_prefix, model_key))
+            config.add_view(cls, attr='mobile_mark_pending', route_name='mobile.{}.mark_pending'.format(route_prefix),
+                            permission='{}.edit'.format(permission_prefix))
 
         # refresh multiple batches (results)
         if cls.results_refreshable:
