@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -65,7 +65,16 @@ class CommonView(View):
         image_url = self.rattail_config.get(
             'tailbone', 'main_image_url',
             default=self.request.static_url('tailbone:static/img/home_logo.png'))
-        return {'image_url': image_url, 'use_buefy': self.get_use_buefy()}
+
+        context = {
+            'image_url': image_url,
+            'use_buefy': self.get_use_buefy(),
+        }
+
+        if self.expose_quickie_search:
+            context['quickie'] = self.get_quickie_context()
+
+        return context
 
     def robots_txt(self):
         """
@@ -189,6 +198,7 @@ class CommonView(View):
     @classmethod
     def _defaults(cls, config):
         rattail_config = config.registry.settings.get('rattail_config')
+        legacy_mobile = cls.legacy_mobile_enabled(rattail_config)
 
         # auto-correct URLs which require trailing slash
         config.add_notfound_view(cls, attr='notfound', append_slash=True)
@@ -203,8 +213,9 @@ class CommonView(View):
         # home
         config.add_route('home', '/')
         config.add_view(cls, attr='home', route_name='home', renderer='/home.mako')
-        config.add_route('mobile.home', '/mobile/')
-        config.add_view(cls, attr='mobile_home', route_name='mobile.home', renderer='/mobile/home.mako')
+        if legacy_mobile:
+            config.add_route('mobile.home', '/mobile/')
+            config.add_view(cls, attr='mobile_home', route_name='mobile.home', renderer='/mobile/home.mako')
 
         # robots.txt
         config.add_route('robots.txt', '/robots.txt')
@@ -213,8 +224,9 @@ class CommonView(View):
         # about
         config.add_route('about', '/about')
         config.add_view(cls, attr='about', route_name='about', renderer='/about.mako')
-        config.add_route('mobile.about', '/mobile/about')
-        config.add_view(cls, attr='about', route_name='mobile.about', renderer='/mobile/about.mako')
+        if legacy_mobile:
+            config.add_route('mobile.about', '/mobile/about')
+            config.add_view(cls, attr='about', route_name='mobile.about', renderer='/mobile/about.mako')
 
         # change db engine
         config.add_tailbone_permission('common', 'common.change_db_engine',
@@ -229,10 +241,15 @@ class CommonView(View):
         config.add_view(cls, attr='change_theme', route_name='change_theme')
 
         # feedback
+        config.add_tailbone_permission('common', 'common.feedback',
+                                       "Send user feedback (to admins) about the app")
         config.add_route('feedback', '/feedback', request_method='POST')
-        config.add_view(cls, attr='feedback', route_name='feedback', renderer='json')
-        config.add_route('mobile.feedback', '/mobile/feedback', request_method='POST')
-        config.add_view(cls, attr='mobile_feedback', route_name='mobile.feedback', renderer='json')
+        config.add_view(cls, attr='feedback', route_name='feedback',
+                        renderer='json', permission='common.feedback')
+        if legacy_mobile:
+            config.add_route('mobile.feedback', '/mobile/feedback', request_method='POST')
+            config.add_view(cls, attr='mobile_feedback', route_name='mobile.feedback',
+                            renderer='json', permission='common.feedback')
 
         # consume batch ID
         config.add_tailbone_permission('common', 'common.consume_batch_id',
