@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2018 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -39,23 +39,27 @@ class CustomerGroupsView(MasterView):
     model_class = model.CustomerGroup
     model_title = "Customer Group"
 
+    labels = {
+        'id': "ID",
+    }
+
+    grid_columns = [
+        'id',
+        'name',
+    ]
+
+    form_fields = [
+        'id',
+        'name',
+    ]
+
     def configure_grid(self, g):
+        super(CustomerGroupsView, self).configure_grid(g)
         g.filters['name'].default_active = True
         g.filters['name'].default_verb = 'contains'
-        g.default_sortkey = 'name'
-        g.configure(
-            include=[
-                g.id.label("ID"),
-                g.name,
-            ],
-            readonly=True)
-
-    def configure_fieldset(self, fs):
-        fs.configure(
-            include=[
-                fs.id.label("ID"),
-                fs.name,
-            ])
+        g.set_sort_defaults('name')
+        g.set_link('id')
+        g.set_link('name')
 
     def before_delete(self, group):
         # First remove customer associations.
