@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2019 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,101 +26,20 @@ Message Views
 
 from __future__ import unicode_literals, absolute_import
 
-import json
-import pytz
+import six
 
-from rattail import enum
 from rattail.db import model
 from rattail.time import localtime
 
-import formalchemy
-from formalchemy.helpers import text_field
+import colander
+from deform import widget as dfwidget
 from pyramid import httpexceptions
-from webhelpers.html import tags, HTML
+from webhelpers2.html import tags, HTML
 
-from tailbone import forms
+# from tailbone import forms
 from tailbone.db import Session
 from tailbone.views import MasterView
-
-
-class SubjectFieldRenderer(formalchemy.FieldRenderer):
-
-    def render_readonly(self, **kwargs):
-        subject = self.raw_value
-        if not subject:
-            return ''
-        return tags.link_to(subject, self.request.route_url('messages.view', uuid=self.field.parent.model.uuid))
-
-
-class SenderFieldRenderer(forms.renderers.UserFieldRenderer):
-
-    def render_readonly(self, **kwargs):
-        sender = self.raw_value
-        if sender is self.request.user:
-            return 'you'
-        return super(SenderFieldRenderer, self).render_readonly(**kwargs)
-
-
-class RecipientsField(formalchemy.Field):
-    """
-    Custom field for recipients, used when sending new messages.
-    """
-    is_collection = True
-
-    def sync(self):
-        if not self.is_readonly():
-            message = self.parent.model
-            for uuid in self._deserialize():
-                user = Session.query(model.User).get(uuid)
-                if user:
-                    message.add_recipient(user, status=enum.MESSAGE_STATUS_INBOX)
-
-
-class RecipientsFieldRenderer(formalchemy.FieldRenderer):
-
-    def render(self, **kwargs):
-        uuids = self.value
-        value = ','.join(uuids) if uuids else ''
-        return text_field(self.name, value=value, **kwargs)
-
-    def deserialize(self):
-        value = self.params.getone(self.name).split(',')
-        value = [uuid.strip() for uuid in value]
-        value = set([uuid for uuid in value if uuid])
-        return value
-
-    def render_readonly(self, **kwargs):
-        recipients = self.raw_value
-        if not recipients:
-            return ''
-        recips = [r for r in recipients if r.recipient is not self.request.user]
-        recips = sorted([r.recipient.display_name for r in recips])
-        if len(recips) < len(recipients):
-            recips.insert(0, 'you')
-        max_display = 5
-        if len(recips) > max_display:
-            basic = HTML.literal("{}, ".format(', '.join(recips[:max_display-1])))
-            more = tags.link_to("({} more)".format(len(recips[max_display-1:])), '#', class_='more')
-            everyone = HTML.tag('span', class_='everyone', c=', '.join(recips[max_display-1:]))
-            return basic + more + everyone
-        return ', '.join(recips)
-
-
-class TerseRecipientsFieldRenderer(formalchemy.FieldRenderer):
-
-    def render_readonly(self, **kwargs):
-        recipients = self.raw_value
-        if not recipients:
-            return ''
-        message = self.field.parent.model
-        recips = [r for r in recipients if r.recipient is not self.request.user]
-        recips = sorted([r.recipient.display_name for r in recips])
-        if len(recips) < len(recipients) and (
-                message.sender is not self.request.user or not recips):
-            recips.insert(0, 'you')
-        if len(recips) < 5:
-            return ', '.join(recips)
-        return "{}, ...".format(', '.join(recips[:4]))
+from tailbone.util import raw_datetime
 
 
 class MessagesView(MasterView):
@@ -134,19 +53,34 @@ class MessagesView(MasterView):
     replying = False
     reply_header_sent_format = '%a %d %b %Y at %I:%M %p'
 
+    grid_columns = [
+        'subject',
+        'sender',
+        'recipients',
+        'sent',
+    ]
+
+    form_fields = [
+        'sender',
+        'recipients',
+        'sent',
+        'subject',
+        'body',
+    ]
+
     def get_index_title(self):
         if self.listing:
             return self.index_title
         if self.viewing:
             message = self.get_instance()
             recipient = self.get_recipient(message)
-            if recipient and recipient.status == enum.MESSAGE_STATUS_ARCHIVE:
+            if recipient and recipient.status == self.enum.MESSAGE_STATUS_ARCHIVE:
                 return "Message Archive"
             elif not recipient:
                 return "Sent Messages"
         return "Message Inbox"
 
-    def get_index_url(self):
+    def get_index_url(self, **kwargs):
         # not really used, but necessary to make certain other code happy
         return self.request.route_url('messages.inbox')
 
@@ -177,7 +111,7 @@ class MessagesView(MasterView):
                       .filter(model.MessageRecipient.recipient == self.request.user)
 
     def configure_grid(self, g):
-
+        
         g.joiners['sender'] = lambda q: q.join(model.User, model.User.uuid == model.Message.sender_uuid).outerjoin(model.Person)
         g.filters['sender'] = g.make_filter('sender', model.Person.display_name,
                                             default_active=True, default_verb='contains')
@@ -186,67 +120,150 @@ class MessagesView(MasterView):
         g.filters['subject'].default_active = True
         g.filters['subject'].default_verb = 'contains'
 
-        g.default_sortkey = 'sent'
-        g.default_sortdir = 'desc'
+        g.set_sort_defaults('sent', 'desc')
 
-        g.configure(
-            include=[
-                g.subject.with_renderer(SubjectFieldRenderer),
-                g.sender.with_renderer(SenderFieldRenderer).label("From"),
-                g.recipients.with_renderer(TerseRecipientsFieldRenderer).label("To"),
-                g.sent,
-            ],
-            readonly=True)
+        g.set_renderer('sent', self.render_sent)
+        g.set_renderer('sender', self.render_sender)
+        g.set_renderer('recipients', self.render_recipients)
 
-    def row_attrs(self, row, i):
-        recip = self.get_recipient(row)
-        if recip:
-            return {'data-uuid': recip.uuid}
-        return {}
+        g.set_link('subject')
 
-    def make_form(self, instance, **kwargs):
-        form = super(MessagesView, self).make_form(instance, **kwargs)
-        if self.creating:
-            form.id = 'new-message'
-            form.cancel_url = self.request.get_referrer(default=self.request.route_url('messages.inbox'))
-        return form
+        g.set_label('sender', "From")
+        g.set_label('recipients', "To")
+
+    def render_sent(self, message, column_name):
+        return raw_datetime(self.rattail_config, message.sent)
+
+    def render_sender(self, message, field):
+        sender = message.sender
+        if sender is self.request.user:
+            return 'you'
+        return six.text_type(sender)
+
+    def render_subject_bold(self, message, field):
+        if not message.subject:
+            return ""
+        return HTML.tag('span', c=message.subject, style='font-weight: bold;')
+
+    def render_recipients(self, message, column_name):
+        recipients = message.recipients
+        if recipients:
+            recips = [r for r in recipients if r.recipient is not self.request.user]
+            recips = sorted([r.recipient.display_name for r in recips])
+            if len(recips) < len(recipients) and (
+                    message.sender is not self.request.user or not recips):
+                recips.insert(0, "you")
+            if len(recips) < 5:
+                return ", ".join(recips)
+            return "{}, ...".format(', '.join(recips[:4]))
+        return ""
+
+    def render_recipients_full(self, message, field):
+        recipients = message.recipients
+        if not recipients:
+            return ""
+
+        use_buefy = self.get_use_buefy()
+
+        # remove current user from displayed list, even if they're a recipient
+        recips = [r for r in recipients
+                  if r.recipient is not self.request.user]
+
+        # sort recipients by display name
+        recips = sorted([r.recipient.display_name for r in recips])
+
+        # if we *did* remove current user from list, insert them at front of list
+        if len(recips) < len(recipients) and (
+                message.sender is not self.request.user or not recips):
+            recips.insert(0, 'you')
+
+        # we only want to show the first 5 recipients by default, with
+        # client-side JS allowing the user to view all if they want
+        max_display = 5
+        if len(recips) > max_display:
+            if use_buefy:
+                basic = HTML.tag('span', c="{}, ".format(', '.join(recips[:max_display-1])))
+                more = tags.link_to("({} more)".format(len(recips[max_display-1:])), '#', **{
+                    'v-show': '!showingAllRecipients',
+                    '@click.prevent': 'showMoreRecipients()',
+                })
+                everyone = HTML.tag('span', c=', '.join(recips[max_display-1:]), **{
+                    'v-show': 'showingAllRecipients',
+                    '@click': 'hideMoreRecipients()',
+                    'class_': 'everyone',
+                })
+                return HTML.tag('div', c=[basic, more, everyone])
+            else:
+                basic = HTML.literal("{}, ".format(', '.join(recips[:max_display-1])))
+                more = tags.link_to("({} more)".format(len(recips[max_display-1:])), '#', class_='more')
+                everyone = HTML.tag('span', class_='everyone', c=', '.join(recips[max_display-1:]))
+                return basic + more + everyone
+
+        # show the full list if there are few enough recipients for that
+        return ', '.join(recips)
+
+    # TODO!!
+    # def make_form(self, instance, **kwargs):
+    #     form = super(MessagesView, self).make_form(instance, **kwargs)
+    #     if self.creating:
+    #         form.id = 'new-message'
+    #         form.cancel_url = self.request.get_referrer(default=self.request.route_url('messages.inbox'))
+    #         form.create_label = "Send Message"
+    #     return form
+
+    def configure_form(self, f):
+        super(MessagesView, self).configure_form(f)
+        use_buefy = self.get_use_buefy()
+
+        f.submit_label = "Send Message"
+
+        if not use_buefy:
+            # we have custom logic to disable submit button
+            f.auto_disable = False
+            f.auto_disable_save = False
 
-    def configure_fieldset(self, fs):
         # TODO: A fair amount of this still seems hacky...
 
+        f.set_renderer('sender', self.render_sender)
+        f.set_label('sender', "From")
+
+        f.set_type('sent', 'datetime')
+
+        # recipients
+        f.set_renderer('recipients', self.render_recipients_full)
+        f.set_label('recipients', "To")
+
+        # subject
+        if use_buefy:
+            f.set_renderer('subject', self.render_subject_bold)
+            if self.creating:
+                f.set_widget('subject', dfwidget.TextInputWidget(
+                    placeholder="please enter a subject",
+                    autocomplete='off'))
+                f.set_required('subject')
+
+        # body
+        f.set_widget('body', dfwidget.TextAreaWidget(cols=50, rows=15))
+
         if self.creating:
+            f.remove('sender', 'sent')
 
-            # Must create a new 'sender' field so that we can feed it the
-            # current user as default value, but prevent attaching user to the
-            # (new) underlying message instance...ugh
-            fs.append(formalchemy.Field('sender', value=self.request.user,
-                                        renderer=forms.renderers.UserFieldRenderer,
-                                        label="From", readonly=True))
-
-            # Sort of the same thing for recipients, although most of that logic is below.
-            fs.append(RecipientsField('recipients', label="To", renderer=RecipientsFieldRenderer))
-
-            fs.configure(include=[
-                fs.sender,
-                fs.recipients,
-                fs.subject,
-                fs.body.textarea(size='50x15'),
-            ])
-
-            # We'll assign some properties directly on the new message;
-            # apparently that's safe and won't cause it to be committed.
-            # Notably, we can't assign the sender yet.  Also the actual
-            # recipients assignment is handled by that field's sync().
-            message = fs.model
+            # recipients
+            f.insert_after('recipients', 'set_recipients')
+            f.remove('recipients')
+            f.set_node('set_recipients', colander.SchemaNode(colander.Set()))
+            if use_buefy:
+                f.set_widget('set_recipients', RecipientsWidgetBuefy())
+            else:
+                f.set_widget('set_recipients', RecipientsWidget())
+            f.set_label('set_recipients', "To")
 
             if self.replying:
                 old_message = self.get_instance()
-                message.subject = "Re: {}".format(old_message.subject)
-                message.body = self.get_reply_body(old_message)
+                f.set_default('subject', "Re: {}".format(old_message.subject))
+                f.set_default('body', self.get_reply_body(old_message))
 
-                # Determine an initial set of recipients, based on reply
-                # method.  This value will be set to a 'pseudo' field to avoid
-                # touching the new model instance and causing a crap commit.
+                # Determine an initial set of recipients, based on reply method.
 
                 # If replying to all, massage the list a little so that the
                 # current user is not listed, and the sender is listed first.
@@ -256,44 +273,48 @@ class MessagesView(MasterView):
                              if self.filter_reply_recipient(r.recipient)]
                     value = dict(value)
                     value.pop(self.request.user.uuid, None)
-                    value = sorted(value.iteritems(), key=lambda r: r[1])
+                    value = sorted(value.items(), key=lambda r: r[1])
                     value = [r[0] for r in value]
                     if old_message.sender is not self.request.user and old_message.sender.active:
                         value.insert(0, old_message.sender_uuid)
-                    fs.recipients.set(value=value)
+                    f.set_default('set_recipients', ','.join(value))
 
                 # Just a normal reply, to sender only.
                 elif self.filter_reply_recipient(old_message.sender):
-                    fs.recipients.set(value=[old_message.sender.uuid])
+                    f.set_default('set_recipients', old_message.sender.uuid)
 
-                # Set focus to message body instead of recipients, when replying.
-                fs.focus = fs.body
+                # TODO?
+                # # Set focus to message body instead of recipients, when replying.
+                # fs.focus = fs.body
 
         elif self.viewing:
+            f.remove('body')
 
-            # Viewing an existing message is a heck of a lot easier...
-            fs.configure(include=[
-                fs.sender.with_renderer(SenderFieldRenderer).label("From"),
-                fs.recipients.with_renderer(RecipientsFieldRenderer).label("To"),
-                fs.sent,
-                fs.subject,
-            ])
+    def objectify(self, form, data=None):
+        if data is None:
+            data = form.validated
+        message = super(MessagesView, self).objectify(form, data)
 
-    def before_create(self, form):
-        """
-        This is where we must assign the current user as sender for new
-        messages, for now.  I'm still not quite happy with this...
-        """
-        super(MessagesView, self).before_create(form)
-        message = form.fieldset.model
-        message.sender = self.request.user
+        if self.creating:
+            if self.request.user:
+                message.sender = self.request.user
+
+            for uuid in data['set_recipients']:
+                user = self.Session.query(model.User).get(uuid)
+                if user:
+                    message.add_recipient(user, status=self.enum.MESSAGE_STATUS_INBOX)
+
+        return message
+
+    def flash_after_create(self, obj):
+        self.request.session.flash("Message has been sent: {}".format(
+            self.get_instance_title(obj)))
 
     def filter_reply_recipient(self, user):
         return user.active
 
     def get_reply_header(self, message):
-        sent = pytz.utc.localize(message.sent)
-        sent = localtime(self.rattail_config, sent)
+        sent = localtime(self.rattail_config, message.sent, from_utc=True)
         sent = sent.strftime(self.reply_header_sent_format)
         return "On {}, {} wrote:".format(sent, message.sender.person.display_name)
 
@@ -319,12 +340,29 @@ class MessagesView(MasterView):
                 return recipient
 
     def template_kwargs_create(self, **kwargs):
-        kwargs['available_recipients'] = self.get_available_recipients()
-        kwargs['json'] = json
+        use_buefy = self.get_use_buefy()
+
+        recips = self.get_available_recipients()
+        if use_buefy:
+            kwargs['recipient_display_map'] = recips
+        recips = list(recips.items())
+        recips.sort(key=self.recipient_sortkey)
+        kwargs['available_recipients'] = recips
+
         if self.replying:
             kwargs['original_message'] = self.get_instance()
+
+        if use_buefy:
+            kwargs['index_url'] = None
+            kwargs['index_title'] = "New Message"
         return kwargs
 
+    def recipient_sortkey(self, recip):
+        uuid, entry = recip
+        if isinstance(entry, dict):
+            return entry['name']
+        return entry
+
     def get_available_recipients(self):
         """
         Return the full mapping of recipients which may be included in a
@@ -340,8 +378,15 @@ class MessagesView(MasterView):
 
     def template_kwargs_view(self, **kwargs):
         message = kwargs['instance']
-        return {'message': message,
-                'recipient': self.get_recipient(message)}
+        recipient = self.get_recipient(message)
+
+        kwargs['message'] = message
+        kwargs['recipient'] = recipient
+
+        if recipient and recipient.status == self.enum.MESSAGE_STATUS_ARCHIVE:
+            kwargs['index_url'] = self.request.route_url('messages.archive')
+
+        return kwargs
 
     def reply(self):
         """
@@ -373,7 +418,7 @@ class MessagesView(MasterView):
             return self.redirect(self.request.get_referrer(
                 default=self.request.route_url('messages_inbox')))
 
-        new_status = enum.MESSAGE_STATUS_INBOX if dest == 'inbox' else enum.MESSAGE_STATUS_ARCHIVE
+        new_status = self.enum.MESSAGE_STATUS_INBOX if dest == 'inbox' else self.enum.MESSAGE_STATUS_ARCHIVE
         if recipient.status != new_status:
             recipient.status = new_status
         return self.redirect(self.request.route_url('messages.{}'.format(
@@ -387,12 +432,14 @@ class MessagesView(MasterView):
         if self.request.method == 'POST':
             uuids = self.request.POST.get('uuids', '').split(',')
             if uuids:
-                new_status = enum.MESSAGE_STATUS_INBOX if dest == 'inbox' else enum.MESSAGE_STATUS_ARCHIVE
+                new_status = self.enum.MESSAGE_STATUS_INBOX if dest == 'inbox' else self.enum.MESSAGE_STATUS_ARCHIVE
                 for uuid in uuids:
-                    recip = Session.query(model.MessageRecipient).get(uuid) if uuid else None
-                    if recip and recip.recipient is self.request.user:
-                        if recip.status != new_status:
-                            recip.status = new_status
+                    recip = self.Session.query(model.MessageRecipient)\
+                                        .filter(model.MessageRecipient.message_uuid == uuid)\
+                                        .filter(model.MessageRecipient.recipient_uuid == self.request.user.uuid)\
+                                        .first()
+                    if recip and recip.status != new_status:
+                        recip.status = new_status
         route = 'messages.{}'.format('archive' if dest == 'inbox' else 'inbox')
         return self.redirect(self.request.route_url(route))
 
@@ -431,12 +478,12 @@ class InboxView(MessagesView):
     grid_key = 'messages.inbox'
     index_title = "Message Inbox"
 
-    def get_index_url(self):
+    def get_index_url(self, **kwargs):
         return self.request.route_url('messages.inbox')
 
     def query(self, session):
         q = super(InboxView, self).query(session)
-        return q.filter(model.MessageRecipient.status == enum.MESSAGE_STATUS_INBOX)
+        return q.filter(model.MessageRecipient.status == self.enum.MESSAGE_STATUS_INBOX)
 
 
 class ArchiveView(MessagesView):
@@ -447,12 +494,12 @@ class ArchiveView(MessagesView):
     grid_key = 'messages.archive'
     index_title = "Message Archive"
 
-    def get_index_url(self):
+    def get_index_url(self, **kwargs):
         return self.request.route_url('messages.archive')
 
     def query(self, session):
         q = super(ArchiveView, self).query(session)
-        return q.filter(model.MessageRecipient.status == enum.MESSAGE_STATUS_ARCHIVE)
+        return q.filter(model.MessageRecipient.status == self.enum.MESSAGE_STATUS_ARCHIVE)
 
 
 class SentView(MessagesView):
@@ -464,7 +511,7 @@ class SentView(MessagesView):
     checkboxes = False
     index_title = "Sent Messages"
 
-    def get_index_url(self):
+    def get_index_url(self, **kwargs):
         return self.request.route_url('messages.sent')
 
     def query(self, session):
@@ -474,6 +521,49 @@ class SentView(MessagesView):
     def configure_grid(self, g):
         super(SentView, self).configure_grid(g)
         g.filters['sender'].default_active = False
+        g.joiners['recipients'] = lambda q: q.join(model.MessageRecipient)\
+                                             .join(model.User, model.User.uuid == model.MessageRecipient.recipient_uuid)\
+                                             .join(model.Person)
+        g.filters['recipients'] = g.make_filter('recipients', model.Person.display_name,
+                                                default_active=True, default_verb='contains')
+
+
+class RecipientsWidget(dfwidget.TextInputWidget):
+
+    def deserialize(self, field, pstruct):
+        if pstruct is colander.null:
+            return []
+        elif not isinstance(pstruct, six.string_types):
+            raise colander.Invalid(field.schema, "Pstruct is not a string")
+        if self.strip:
+            pstruct = pstruct.strip()
+        if not pstruct:
+            return []
+        return pstruct.split(',')
+
+
+class RecipientsWidgetBuefy(dfwidget.Widget):
+    """
+    Custom "message recipients" widget, for use with Buefy / Vue.js themes.
+    """
+    template = 'message_recipients_buefy'
+
+    def deserialize(self, field, pstruct):
+        if pstruct is colander.null:
+            return colander.null
+        if not isinstance(pstruct, six.string_types):
+            raise colander.Invalid(field.schema, "Pstruct is not a string")
+        if not pstruct:
+            return colander.null
+        pstruct = pstruct.split(',')
+        return pstruct
+
+    def serialize(self, field, cstruct, **kw):
+        if cstruct in (colander.null, None):
+            cstruct = ""
+        template = self.template
+        values = self.get_template_values(field, cstruct, kw)
+        return field.renderer(template, **values)
 
 
 def includeme(config):
