@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2020 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -32,6 +32,7 @@ from sqlalchemy import orm
 
 from rattail.db import model, api
 from rattail.time import localtime
+from rattail.util import OrderedDict
 
 import colander
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
@@ -51,8 +52,15 @@ class PeopleView(MasterView):
     touchable = True
     has_versions = True
     supports_mobile = True
+    bulk_deletable = True
+    is_contact = True
     manage_notes_from_profile_view = False
 
+    labels = {
+        'default_phone': "Phone Number",
+        'default_email': "Email Address",
+    }
+
     grid_columns = [
         'display_name',
         'first_name',
@@ -66,11 +74,12 @@ class PeopleView(MasterView):
         'middle_name',
         'last_name',
         'display_name',
-        'phone',
-        'email',
+        'default_phone',
+        'default_email',
         'address',
         'employee',
         'customers',
+        'members',
         'users',
     ]
 
@@ -146,17 +155,76 @@ class PeopleView(MasterView):
             return not bool(person.user and person.user.username == 'chuck')
         return True
 
+    def delete_instance(self, person):
+        """
+        Supplements the default logic as follows:
+
+        Any customer associations are first deleted for the person.  Once that
+        is complete, deletion continues as per usual.
+        """
+        session = orm.object_session(person)
+
+        # must explicitly remove all CustomerPerson records
+        for cp in list(person._customers):
+            customer = cp.customer
+            session.delete(cp)
+            # session.flush()
+            customer._people.reorder()
+
+        # continue with normal logic
+        super(PeopleView, self).delete_instance(person)
+
+    def touch_instance(self, person):
+        """
+        Supplements the default logic as follows:
+
+        In addition to "touching" the person proper, we also "touch" each
+        contact info record associated with them.
+        """
+        # touch person, as per usual
+        super(PeopleView, self).touch_instance(person)
+
+        def touch(obj):
+            change = model.Change()
+            change.class_name = obj.__class__.__name__
+            change.instance_uuid = obj.uuid
+            change.deleted = False
+            self.Session.add(change)
+
+        # phone numbers
+        for phone in person.phones:
+            touch(phone)
+
+        # email addresses
+        for email in person.emails:
+            touch(email)
+
+        # mailing addresses
+        for address in person.addresses:
+            touch(address)
+
     def configure_common_form(self, f):
         super(PeopleView, self).configure_common_form(f)
+        person = f.model_instance
 
         f.set_label('display_name', "Full Name")
 
+        # TODO: should remove this?
         f.set_readonly('phone')
         f.set_label('phone', "Phone Number")
 
+        f.set_renderer('default_phone', self.render_default_phone)
+        if not self.creating and person.phones:
+            f.set_default('default_phone', person.phones[0].number)
+
+        # TODO: should remove this?
         f.set_readonly('email')
         f.set_label('email', "Email Address")
 
+        f.set_renderer('default_email', self.render_default_email)
+        if not self.creating and person.emails:
+            f.set_default('default_email', person.emails[0].address)
+
         f.set_readonly('address')
         f.set_label('address', "Mailing Address")
 
@@ -174,6 +242,13 @@ class PeopleView(MasterView):
             f.set_readonly('customers')
             f.set_renderer('customers', self.render_customers)
 
+        # members
+        if self.creating:
+            f.remove_field('members')
+        else:
+            f.set_readonly('members')
+            f.set_renderer('members', self.render_members)
+
         # users
         if self.creating:
             f.remove_field('users')
@@ -197,15 +272,30 @@ class PeopleView(MasterView):
         for customer in customers:
             customer = customer.customer
             text = six.text_type(customer)
-            if customer.id:
+            if customer.number:
+                text = "(#{}) {}".format(customer.number, text)
+            elif customer.id:
                 text = "({}) {}".format(customer.id, text)
-            elif customer.number:
-                text = "({}) {}".format(customer.number, text)
             route = '{}customers.view'.format('mobile.' if self.mobile else '')
             url = self.request.route_url(route, uuid=customer.uuid)
             items.append(HTML.tag('li', c=[tags.link_to(text, url)]))
         return HTML.tag('ul', c=items)
 
+    def render_members(self, person, field):
+        members = person.members
+        if not members:
+            return ""
+        items = []
+        for member in members:
+            text = six.text_type(member)
+            if member.number:
+                text = "(#{}) {}".format(member.number, text)
+            elif member.id:
+                text = "({}) {}".format(member.id, text)
+            url = self.request.route_url('members.view', uuid=member.uuid)
+            items.append(HTML.tag('li', c=[tags.link_to(text, url)]))
+        return HTML.tag('ul', c=items)
+
     def render_users(self, person, field):
         use_buefy = self.get_use_buefy()
         users = person.users
@@ -248,6 +338,9 @@ class PeopleView(MasterView):
             'instance': person,
             'instance_title': self.get_instance_title(person),
             'today': localtime(self.rattail_config).date(),
+            'person_data': self.get_context_person(person),
+            'customers_data': self.get_context_customers(person),
+            'members_data': self.get_context_members(person),
             'employee': employee,
             'employee_view_url': self.request.route_url('employees.view', uuid=employee.uuid) if employee else None,
             'employee_history': employee.get_current_history() if employee else None,
@@ -258,6 +351,81 @@ class PeopleView(MasterView):
         template = 'view_profile_buefy' if use_buefy else 'view_profile'
         return self.render_to_response(template, context)
 
+    def get_context_person(self, person):
+        return {
+            'uuid': person.uuid,
+            'first_name': person.first_name,
+            'last_name': person.last_name,
+            'display_name': person.display_name,
+            'view_url': self.get_action_url('view', person),
+            'view_profile_url': self.get_action_url('view_profile', person),
+        }
+
+    def get_context_address(self, address):
+        return {
+            'uuid': address.uuid,
+            'street': address.street,
+            'street2': address.street2,
+            'city': address.city,
+            'state': address.state,
+            'zipcode': address.zipcode,
+            'display': six.text_type(address),
+        }
+
+    def get_context_customers(self, person):
+        data = []
+        for cp in person._customers:
+            customer = cp.customer
+            data.append({
+                'uuid': customer.uuid,
+                'ordinal': cp.ordinal,
+                'id': customer.id,
+                'number': customer.number,
+                'name': customer.name,
+                'view_url': self.request.route_url('customers.view',
+                                                   uuid=customer.uuid),
+                'people': [self.get_context_person(p)
+                           for p in customer.people],
+                'addresses': [self.get_context_address(a)
+                              for a in customer.addresses],
+            })
+        return data
+
+    def get_context_members(self, person):
+        data = OrderedDict()
+
+        for member in person.members:
+            data[member.uuid] = self.get_context_member(member)
+
+        for customer in person.customers:
+            for member in customer.members:
+                if member.uuid not in data:
+                    data[member.uuid] = self.get_context_member(member)
+
+        return list(data.values())
+
+    def get_context_member(self, member):
+        profile_url = None
+        if member.person:
+            profile_url = self.request.route_url('people.view_profile',
+                                                 uuid=member.person_uuid)
+
+        return {
+            'uuid': member.uuid,
+            'number': member.number,
+            'id': member.id,
+            'active': member.active,
+            'joined': six.text_type(member.joined) if member.joined else None,
+            'withdrew': six.text_type(member.withdrew) if member.withdrew else None,
+            'customer_uuid': member.customer_uuid,
+            'customer_name': member.customer.name if member.customer else None,
+            'person_uuid': member.person_uuid,
+            'display': six.text_type(member),
+            'person_display_name': member.person.display_name if member.person else None,
+            'view_url': self.request.route_url('members.view', uuid=member.uuid),
+            'view_profile_url': profile_url,
+        }
+
     def get_context_employee_history(self, employee):
         data = []
         if employee:
