@@ -87,6 +87,7 @@ class ProductsView(MasterView):
     results_downloadable_xlsx = True
 
     labels = {
+        'item_id': "Item ID",
         'upc': "UPC",
         'status_code': "Status",
         'tax1': "Tax 1",
@@ -483,7 +484,7 @@ class ProductsView(MasterView):
             return text
 
         if self.get_use_buefy():
-            kwargs = {'@click.prevent': 'showingPriceHistory_{} = true'.format(typ)}
+            kwargs = {'@click.prevent': 'showPriceHistory_{}()'.format(typ)}
         else:
             kwargs = {'id': 'view-{}-price-history'.format(typ)}
         history = tags.link_to("(view history)", '#', **kwargs)
@@ -494,10 +495,17 @@ class ProductsView(MasterView):
         br = HTML.tag('br')
         return HTML.tag('div', c=[text, br, history])
 
+    def show_price_effective_dates(self):
+        if not self.rattail_config.versioning_enabled():
+            return False
+        return self.rattail_config.getbool(
+            'tailbone', 'products.show_effective_price_dates',
+            default=True)
+
     def render_regular_price(self, product, field):
         text = self.render_price(product, field)
 
-        if text and self.rattail_config.versioning_enabled():
+        if text and self.show_price_effective_dates():
             history = self.get_regular_price_history(product)
             if history:
                 date = localtime(self.rattail_config, history[0]['changed'], from_utc=True).date()
@@ -508,7 +516,7 @@ class ProductsView(MasterView):
     def render_current_price(self, product, field):
         text = self.render_price(product, field)
 
-        if text and self.rattail_config.versioning_enabled():
+        if text and self.show_price_effective_dates():
             history = self.get_current_price_history(product)
             if history:
                 date = localtime(self.rattail_config, history[0]['changed'], from_utc=True).date()
@@ -526,7 +534,7 @@ class ProductsView(MasterView):
     def render_suggested_price(self, product, column):
         text = self.render_price(product, column)
 
-        if text and self.rattail_config.versioning_enabled():
+        if text and self.show_price_effective_dates():
             history = self.get_suggested_price_history(product)
             if history:
                 date = localtime(self.rattail_config, history[0]['changed'], from_utc=True).date()
@@ -973,8 +981,63 @@ class ProductsView(MasterView):
             return ""
         return pretty_quantity(value)
 
+    def price_history(self):
+        """
+        AJAX view for fetching various types of price history for a product.
+        """
+        product = self.get_instance()
+
+        typ = self.request.params.get('type', 'regular')
+        assert typ in ('regular', 'current', 'suggested')
+
+        getter = getattr(self, 'get_{}_price_history'.format(typ))
+        data = getter(product)
+
+        # make some data JSON-friendly
+        jsdata = []
+        for history in data:
+            history = dict(history)
+            price = history['price']
+            history['price'] = float(price)
+            history['price_display'] = "${:0.2f}".format(price)
+            changed = localtime(self.rattail_config, history['changed'], from_utc=True)
+            history['changed'] = six.text_type(changed)
+            history['changed_display_html'] = raw_datetime(self.rattail_config, changed)
+            user = history.pop('changed_by')
+            history['changed_by_uuid'] = user.uuid
+            history['changed_by_display'] = six.text_type(user)
+            jsdata.append(history)
+        return jsdata
+
+    def cost_history(self):
+        """
+        AJAX view for fetching cost history for a product.
+        """
+        product = self.get_instance()
+        data = self.get_cost_history(product)
+
+        # make some data JSON-friendly
+        jsdata = []
+        for history in data:
+            history = dict(history)
+            cost = history['cost']
+            if cost is not None:
+                history['cost'] = float(cost)
+                history['cost_display'] = "${:0.2f}".format(cost)
+            else:
+                history['cost_display'] = None
+            changed = localtime(self.rattail_config, history['changed'], from_utc=True)
+            history['changed'] = six.text_type(changed)
+            history['changed_display_html'] = raw_datetime(self.rattail_config, changed)
+            user = history.pop('changed_by')
+            history['changed_by_uuid'] = user.uuid
+            history['changed_by_display'] = six.text_type(user)
+            jsdata.append(history)
+        return jsdata
+
     def template_kwargs_view(self, **kwargs):
         product = kwargs['instance']
+        use_buefy = self.get_use_buefy()
 
         # TODO: pretty sure this is no longer needed?  guess we'll find out
         # kwargs['image'] = False
@@ -998,7 +1061,10 @@ class ProductsView(MasterView):
         if self.rattail_config.versioning_enabled() and self.has_perm('versions'):
 
             # regular price
-            data = self.get_regular_price_history(product)
+            if use_buefy:
+                data = []       # defer fetching until user asks for it
+            else:
+                data = self.get_regular_price_history(product)
             grid = grids.Grid('products.regular_price_history', data,
                               request=self.request,
                               columns=[
@@ -1012,7 +1078,10 @@ class ProductsView(MasterView):
             kwargs['regular_price_history_grid'] = grid
 
             # current price
-            data = self.get_current_price_history(product)
+            if use_buefy:
+                data = []       # defer fetching until user asks for it
+            else:
+                data = self.get_current_price_history(product)
             grid = grids.Grid('products.current_price_history', data,
                               request=self.request,
                               columns=[
@@ -1030,7 +1099,10 @@ class ProductsView(MasterView):
             kwargs['current_price_history_grid'] = grid
 
             # suggested price
-            data = self.get_suggested_price_history(product)
+            if use_buefy:
+                data = []       # defer fetching until user asks for it
+            else:
+                data = self.get_suggested_price_history(product)
             grid = grids.Grid('products.suggested_price_history', data,
                               request=self.request,
                               columns=[
@@ -1044,7 +1116,10 @@ class ProductsView(MasterView):
             kwargs['suggested_price_history_grid'] = grid
 
             # cost history
-            data = self.get_cost_history(product)
+            if use_buefy:
+                data = []       # defer fetching until user asks for it
+            else:
+                data = self.get_cost_history(product)
             grid = grids.Grid('products.cost_history', data,
                               request=self.request,
                               columns=[
@@ -1651,11 +1726,14 @@ class ProductsView(MasterView):
 
     @classmethod
     def _product_defaults(cls, config):
+        rattail_config = config.registry.settings.get('rattail_config')
         route_prefix = cls.get_route_prefix()
         url_prefix = cls.get_url_prefix()
+        instance_url_prefix = cls.get_instance_url_prefix()
         template_prefix = cls.get_template_prefix()
         permission_prefix = cls.get_permission_prefix()
         model_title = cls.get_model_title()
+        legacy_mobile = cls.legacy_mobile_enabled(rattail_config)
 
         # print labels
         config.add_tailbone_permission('products', 'products.print_labels',
@@ -1682,9 +1760,24 @@ class ProductsView(MasterView):
         config.add_route('products.image', '/products/{uuid}/image')
         config.add_view(cls, attr='image', route_name='products.image')
 
+        # price history
+        config.add_route('{}.price_history'.format(route_prefix), '{}/price-history'.format(instance_url_prefix),
+                         request_method='GET')
+        config.add_view(cls, attr='price_history', route_name='{}.price_history'.format(route_prefix),
+                        renderer='json',
+                        permission='{}.versions'.format(permission_prefix))
+
+        # cost history
+        config.add_route('{}.cost_history'.format(route_prefix), '{}/cost-history'.format(instance_url_prefix),
+                         request_method='GET')
+        config.add_view(cls, attr='cost_history', route_name='{}.cost_history'.format(route_prefix),
+                        renderer='json',
+                        permission='{}.versions'.format(permission_prefix))
+
         # mobile quick lookup
-        config.add_route('mobile.products.quick_lookup', '/mobile/products/quick-lookup')
-        config.add_view(cls, attr='mobile_quick_lookup', route_name='mobile.products.quick_lookup')
+        if legacy_mobile:
+            config.add_route('mobile.products.quick_lookup', '/mobile/products/quick-lookup')
+            config.add_view(cls, attr='mobile_quick_lookup', route_name='mobile.products.quick_lookup')
 
 
 class ProductsAutocomplete(AutocompleteView):
