@@ -520,7 +520,7 @@ class PurchasingBatchView(BatchMasterView):
         elif purchase.status == self.enum.PURCHASE_STATUS_RECEIVED:
             date = purchase.date_received
             total = purchase.invoice_total
-        return '{} for ${:0,.2f} ({})'.format(date, total, purchase.department or purchase.buyer)
+        return '{} for ${:0,.2f} ({})'.format(date, total or 0, purchase.department or purchase.buyer)
 
     def get_batch_kwargs(self, batch, mobile=False):
         kwargs = super(PurchasingBatchView, self).get_batch_kwargs(batch, mobile=mobile)
@@ -975,11 +975,13 @@ class PurchasingBatchView(BatchMasterView):
 
     @classmethod
     def _purchasing_defaults(cls, config):
+        rattail_config = config.registry.settings.get('rattail_config')
         route_prefix = cls.get_route_prefix()
         url_prefix = cls.get_url_prefix()
         permission_prefix = cls.get_permission_prefix()
         model_key = cls.get_model_key()
         model_title = cls.get_model_title()
+        legacy_mobile = cls.legacy_mobile_enabled(rattail_config)
 
         # eligible purchases (AJAX)
         config.add_route('{}.eligible_purchases'.format(route_prefix), '{}/eligible-purchases'.format(url_prefix))
@@ -987,7 +989,7 @@ class PurchasingBatchView(BatchMasterView):
                         renderer='json', permission='{}.view'.format(permission_prefix))
 
         # add new product
-        if cls.supports_new_product:
+        if legacy_mobile and cls.supports_new_product:
             config.add_tailbone_permission(permission_prefix, '{}.new_product'.format(permission_prefix),
                                            "Create new Product when adding row to {}".format(model_title))
             config.add_route('mobile.{}.new_product'.format(route_prefix), '{}/{{{}}}/new-product'.format(url_prefix, model_key))
