@@ -38,9 +38,10 @@ from rattail.files import resource_path
 from rattail.time import localtime
 from rattail.reporting import get_report_handler
 from rattail.threads import Thread
-from rattail.util import simple_error
+from rattail.util import simple_error, OrderedDict
 
 import colander
+from deform import widget as dfwidget
 from mako.template import Template
 from pyramid.response import Response
 from webhelpers2.html import HTML
@@ -251,9 +252,10 @@ class ReportOutputView(ExportMasterView):
         if not params:
             return ""
 
-        # TODO: should sort these, perhaps according to Report definition?
         params = [{'key': key, 'value': value}
                   for key, value in params.items()]
+        # TODO: should sort these according to true Report definition instead?
+        params.sort(key=lambda param: param['key'])
 
         route_prefix = self.get_route_prefix()
         g = grids.Grid(
@@ -295,9 +297,13 @@ class GenerateReport(View):
 
         # handler is responsible for determining which report types are valid
         reports = self.handler.get_reports()
+        if isinstance(reports, OrderedDict):
+            sorted_reports = list(reports)
+        else:
+            sorted_reports = sorted(reports, key=lambda k: reports[k].name)
 
         # make form to accept user choice of report type
-        schema = NewReport().bind(valid_report_types=list(reports))
+        schema = NewReport().bind(valid_report_types=sorted_reports)
         form = forms.Form(schema=schema, request=self.request, use_buefy=use_buefy)
         form.submit_label = "Continue"
         form.cancel_url = self.request.route_url('report_output')
@@ -323,6 +329,7 @@ class GenerateReport(View):
             'form': form,
             'dform': form.make_deform_form(),
             'reports': reports,
+            'sorted_reports': sorted_reports,
             'report_descriptions': dict([(r.type_key, r.__doc__)
                                          for r in reports.values()]),
             'use_form': self.rattail_config.getbool('tailbone', 'reporting.choosing_uses_form',
@@ -342,7 +349,9 @@ class GenerateReport(View):
         report_params = report.make_params(Session())
 
         NODE_TYPES = {
+            bool: colander.Boolean,
             datetime.date: colander.Date,
+            'decimal': colander.Decimal,
         }
 
         schema = colander.Schema()
@@ -352,10 +361,19 @@ class GenerateReport(View):
             node_type = NODE_TYPES.get(param.type, colander.String)
             node = colander.SchemaNode(typ=node_type(), name=param.name)
 
+            # maybe setup choices, if appropriate
+            if param.type == 'choice':
+                node.widget = dfwidget.SelectWidget(
+                    values=report.get_choices(param.name, Session()))
+
             # allow empty value if param is optional
             if not param.required:
                 node.missing = colander.null
 
+            # maybe set default value
+            if hasattr(param, 'default'):
+                node.default = param.default
+
             schema.add(node)
 
         form = forms.Form(schema=schema, request=self.request,
