@@ -42,6 +42,7 @@ from tailbone import forms
 from tailbone.db import Session
 from tailbone.views import MasterView
 from tailbone.views.principal import PrincipalMasterView, PermissionsRenderer
+from tailbone.config import protected_usernames
 
 
 class UsersView(PrincipalMasterView):
@@ -57,6 +58,7 @@ class UsersView(PrincipalMasterView):
         'username',
         'person',
         'active',
+        'local_only',
     ]
 
     form_fields = [
@@ -84,6 +86,7 @@ class UsersView(PrincipalMasterView):
     merge_coalesce_fields = [
         'person_uuid',
         'person_name',
+        'active',
     ]
     merge_fields = merge_additive_fields + [
         'uuid',
@@ -91,7 +94,6 @@ class UsersView(PrincipalMasterView):
         'person_uuid',
         'person_name',
         'role_count',
-        'active',
     ]
 
     def query(self, session):
@@ -138,9 +140,9 @@ class UsersView(PrincipalMasterView):
         user is "root".  But if the given user is not protected, this simply
         returns ``True``.
         """
-        if self.user_is_protected(user):
-            return self.request.is_root
-        return True
+        if self.request.is_root:
+            return True
+        return not self.user_is_protected(user)
 
     def deletable_instance(self, user):
         """
@@ -148,9 +150,9 @@ class UsersView(PrincipalMasterView):
         user is "root".  But if the given user is not protected, this simply
         returns ``True``.
         """
-        if self.user_is_protected(user):
-            return self.request.is_root
-        return True
+        if self.request.is_root:
+            return True
+        return not self.user_is_protected(user)
 
     def user_is_protected(self, user):
         """
@@ -164,8 +166,7 @@ class UsersView(PrincipalMasterView):
         "root", otherwise will return ``False``.
         """
         if not hasattr(self, 'protected_usernames'):
-            self.protected_usernames = self.rattail_config.getlist(
-                'tailbone', 'protected_usernames')
+            self.protected_usernames = protected_usernames(self.rattail_config)
         if self.protected_usernames and user.username in self.protected_usernames:
             return True
         return False
